@@ -46,61 +46,81 @@
 #'
 #' @inherit shared_params return
 #'
-#' @examples
-#' library(teal.widgets)
+#' @section Decorating Module:
 #'
+#' This module generates the following objects, which can be modified in place using decorators:
+#' - `plot` (`ggplot`)
+#'
+#' A Decorator is applied to the specific output using a named list of `teal_transform_module` objects.
+#' The name of this list corresponds to the name of the output to which the decorator is applied.
+#' See code snippet below:
+#'
+#' ```
+#' tm_g_bivariate(
+#'    ..., # arguments for module
+#'    decorators = list(
+#'      plot = teal_transform_module(...) # applied to the `plot` output
+#'    )
+#' )
+#' ```
+#'
+#' For additional details and examples of decorators, refer to the vignette
+#' `vignette("decorate-module-output", package = "teal.modules.general")`.
+#'
+#' To learn more please refer to the vignette
+#' `vignette("transform-module-output", package = "teal")` or the [`teal::teal_transform_module()`] documentation.
+#'
+#'
+#' @examplesShinylive
+#' library(teal.modules.general)
+#' interactive <- function() TRUE
+#' {{ next_example }}
+#' @examples
 #' # general data example
 #' data <- teal_data()
 #' data <- within(data, {
 #'   require(nestcolor)
 #'   CO2 <- data.frame(CO2)
 #' })
-#' datanames(data) <- c("CO2")
-#' join_keys(data) <- default_cdisc_join_keys[datanames(data)]
 #'
 #' app <- init(
 #'   data = data,
-#'   modules = modules(
-#'     tm_g_bivariate(
-#'       x = data_extract_spec(
-#'         dataname = "CO2",
-#'         select = select_spec(
-#'           label = "Select variable:",
-#'           choices = variable_choices(data[["CO2"]]),
-#'           selected = "conc",
-#'           fixed = FALSE
-#'         )
-#'       ),
-#'       y = data_extract_spec(
-#'         dataname = "CO2",
-#'         select = select_spec(
-#'           label = "Select variable:",
-#'           choices = variable_choices(data[["CO2"]]),
-#'           selected = "uptake",
-#'           multiple = FALSE,
-#'           fixed = FALSE
-#'         )
-#'       ),
-#'       row_facet = data_extract_spec(
-#'         dataname = "CO2",
-#'         select = select_spec(
-#'           label = "Select variable:",
-#'           choices = variable_choices(data[["CO2"]]),
-#'           selected = "Type",
-#'           fixed = FALSE
-#'         )
-#'       ),
-#'       col_facet = data_extract_spec(
-#'         dataname = "CO2",
-#'         select = select_spec(
-#'           label = "Select variable:",
-#'           choices = variable_choices(data[["CO2"]]),
-#'           selected = "Treatment",
-#'           fixed = FALSE
-#'         )
-#'       ),
-#'       ggplot2_args = ggplot2_args(
-#'         labs = list(subtitle = "Plot generated by Bivariate Module")
+#'   modules = tm_g_bivariate(
+#'     x = data_extract_spec(
+#'       dataname = "CO2",
+#'       select = select_spec(
+#'         label = "Select variable:",
+#'         choices = variable_choices(data[["CO2"]]),
+#'         selected = "conc",
+#'         fixed = FALSE
+#'       )
+#'     ),
+#'     y = data_extract_spec(
+#'       dataname = "CO2",
+#'       select = select_spec(
+#'         label = "Select variable:",
+#'         choices = variable_choices(data[["CO2"]]),
+#'         selected = "uptake",
+#'         multiple = FALSE,
+#'         fixed = FALSE
+#'       )
+#'     ),
+#'     row_facet = data_extract_spec(
+#'       dataname = "CO2",
+#'       select = select_spec(
+#'         label = "Select variable:",
+#'         choices = variable_choices(data[["CO2"]]),
+#'         selected = "Type",
+#'         fixed = FALSE
+#'       )
+#'     ),
+#'     col_facet = data_extract_spec(
+#'       dataname = "CO2",
+#'       select = select_spec(
+#'         label = "Select variable:",
+#'         choices = variable_choices(data[["CO2"]]),
+#'         selected = "Treatment",
+#'         fixed = FALSE
 #'       )
 #'     )
 #'   )
@@ -109,59 +129,57 @@
 #'   shinyApp(app$ui, app$server)
 #' }
 #'
-#'
+#' @examplesShinylive
+#' library(teal.modules.general)
+#' interactive <- function() TRUE
+#' {{ next_example }}
+#' @examples
 #' # CDISC data example
 #' data <- teal_data()
 #' data <- within(data, {
 #'   require(nestcolor)
-#'   ADSL <- rADSL
+#'   ADSL <- teal.data::rADSL
 #' })
-#' datanames(data) <- c("ADSL")
-#' join_keys(data) <- default_cdisc_join_keys[datanames(data)]
+#' join_keys(data) <- default_cdisc_join_keys[names(data)]
 #'
 #' app <- init(
 #'   data = data,
-#'   modules = modules(
-#'     tm_g_bivariate(
-#'       x = data_extract_spec(
-#'         dataname = "ADSL",
-#'         select = select_spec(
-#'           label = "Select variable:",
-#'           choices = variable_choices(data[["ADSL"]]),
-#'           selected = "AGE",
-#'           fixed = FALSE
-#'         )
-#'       ),
-#'       y = data_extract_spec(
-#'         dataname = "ADSL",
-#'         select = select_spec(
-#'           label = "Select variable:",
-#'           choices = variable_choices(data[["ADSL"]]),
-#'           selected = "SEX",
-#'           multiple = FALSE,
-#'           fixed = FALSE
-#'         )
-#'       ),
-#'       row_facet = data_extract_spec(
-#'         dataname = "ADSL",
-#'         select = select_spec(
-#'           label = "Select variable:",
-#'           choices = variable_choices(data[["ADSL"]]),
-#'           selected = "ARM",
-#'           fixed = FALSE
-#'         )
-#'       ),
-#'       col_facet = data_extract_spec(
-#'         dataname = "ADSL",
-#'         select = select_spec(
-#'           label = "Select variable:",
-#'           choices = variable_choices(data[["ADSL"]]),
-#'           selected = "COUNTRY",
-#'           fixed = FALSE
-#'         )
-#'       ),
-#'       ggplot2_args = ggplot2_args(
-#'         labs = list(subtitle = "Plot generated by Bivariate Module")
+#'   modules = tm_g_bivariate(
+#'     x = data_extract_spec(
+#'       dataname = "ADSL",
+#'       select = select_spec(
+#'         label = "Select variable:",
+#'         choices = variable_choices(data[["ADSL"]]),
+#'         selected = "AGE",
+#'         fixed = FALSE
+#'       )
+#'     ),
+#'     y = data_extract_spec(
+#'       dataname = "ADSL",
+#'       select = select_spec(
+#'         label = "Select variable:",
+#'         choices = variable_choices(data[["ADSL"]]),
+#'         selected = "SEX",
+#'         multiple = FALSE,
+#'         fixed = FALSE
+#'       )
+#'     ),
+#'     row_facet = data_extract_spec(
+#'       dataname = "ADSL",
+#'       select = select_spec(
+#'         label = "Select variable:",
+#'         choices = variable_choices(data[["ADSL"]]),
+#'         selected = "ARM",
+#'         fixed = FALSE
+#'       )
+#'     ),
+#'     col_facet = data_extract_spec(
+#'       dataname = "ADSL",
+#'       select = select_spec(
+#'         label = "Select variable:",
+#'         choices = variable_choices(data[["ADSL"]]),
+#'         selected = "COUNTRY",
+#'         fixed = FALSE
 #'       )
 #'     )
 #'   )
@@ -192,8 +210,10 @@ tm_g_bivariate <- function(label = "Bivariate Plots",
                            ggtheme = c("gray", "bw", "linedraw", "light", "dark", "minimal", "classic", "void"),
                            ggplot2_args = teal.widgets::ggplot2_args(),
                            pre_output = NULL,
-                           post_output = NULL) {
-  logger::log_info("Initializing tm_g_bivariate")
+                           post_output = NULL,
+                           transformators = list(),
+                           decorators = list()) {
+  message("Initializing tm_g_bivariate")
 
   # Normalize the parameters
   if (inherits(x, "data_extract_spec")) x <- list(x)
@@ -272,6 +292,8 @@ tm_g_bivariate <- function(label = "Bivariate Plots",
 
   checkmate::assert_multi_class(pre_output, c("shiny.tag", "shiny.tag.list", "html"), null.ok = TRUE)
   checkmate::assert_multi_class(post_output, c("shiny.tag", "shiny.tag.list", "html"), null.ok = TRUE)
+
+  assert_decorators(decorators, "plot")
   # End of assertions
 
   # Make UI args
@@ -288,17 +310,20 @@ tm_g_bivariate <- function(label = "Bivariate Plots",
     size = size
   )
 
-  module(
+  ans <- module(
     label = label,
     server = srv_g_bivariate,
     ui = ui_g_bivariate,
     ui_args = args,
     server_args = c(
       data_extract_list,
-      list(plot_height = plot_height, plot_width = plot_width, ggplot2_args = ggplot2_args)
+      list(plot_height = plot_height, plot_width = plot_width, ggplot2_args = ggplot2_args, decorators = decorators)
     ),
+    transformators = transformators,
     datanames = teal.transform::get_extract_datanames(data_extract_list)
   )
+  attr(ans, "teal_bookmarkable") <- TRUE
+  ans
 }
 
 # UI function for the bivariate module
@@ -313,7 +338,7 @@ ui_g_bivariate <- function(id, ...) {
     output = teal.widgets::white_small_well(
       tags$div(teal.widgets::plot_with_settings_ui(id = ns("myplot")))
     ),
-    encoding = div(
+    encoding = tags$div(
       ### Reporter
       teal.reporter::simple_reporter_ui(ns("simple_reporter")),
       ###
@@ -343,14 +368,15 @@ ui_g_bivariate <- function(id, ...) {
           justified = TRUE
         )
       ),
+      ui_decorate_teal_data(ns("decorator"), decorators = select_decorators(args$decorators, "plot")),
       if (!is.null(args$row_facet) || !is.null(args$col_facet)) {
-        div(
+        tags$div(
           class = "data-extract-box",
           tags$label("Facetting"),
           shinyWidgets::switchInput(inputId = ns("facetting"), value = args$facet, size = "mini"),
           conditionalPanel(
             condition = paste0("input['", ns("facetting"), "']"),
-            div(
+            tags$div(
               if (!is.null(args$row_facet)) {
                 teal.transform::data_extract_ui(
                   id = ns("row_facet"),
@@ -375,13 +401,13 @@ ui_g_bivariate <- function(id, ...) {
       },
       if (args$color_settings) {
         # Put a grey border around the coloring settings
-        div(
+        tags$div(
           class = "data-extract-box",
           tags$label("Color settings"),
           shinyWidgets::switchInput(inputId = ns("coloring"), value = TRUE, size = "mini"),
           conditionalPanel(
             condition = paste0("input['", ns("coloring"), "']"),
-            div(
+            tags$div(
               teal.transform::data_extract_ui(
                 id = ns("color"),
                 label = "Outline color by variable",
@@ -394,7 +420,7 @@ ui_g_bivariate <- function(id, ...) {
                 data_extract_spec = args$fill,
                 is_single_dataset = is_single_dataset_value
               ),
-              div(
+              tags$div(
                 id = ns("size_settings"),
                 teal.transform::data_extract_ui(
                   id = ns("size"),
@@ -434,7 +460,6 @@ ui_g_bivariate <- function(id, ...) {
       )
     ),
     forms = tagList(
-      teal.widgets::verbatim_popup_ui(ns("warning"), button_label = "Show Warnings"),
       teal.widgets::verbatim_popup_ui(ns("rcode"), "Show R code")
     ),
     pre_output = args$pre_output,
@@ -457,12 +482,17 @@ srv_g_bivariate <- function(id,
                             size,
                             plot_height,
                             plot_width,
-                            ggplot2_args) {
+                            ggplot2_args,
+                            decorators) {
   with_reporter <- !missing(reporter) && inherits(reporter, "Reporter")
   with_filter <- !missing(filter_panel_api) && inherits(filter_panel_api, "FilterPanelAPI")
   checkmate::assert_class(data, "reactive")
   checkmate::assert_class(isolate(data()), "teal_data")
   moduleServer(id, function(input, output, session) {
+    teal.logger::log_shiny_input_changes(input, namespace = "teal.modules.general")
+
+    ns <- session$ns
+
     data_extract <- list(
       x = x, y = y, row_facet = row_facet, col_facet = col_facet,
       color = color, fill = fill, size = size
@@ -520,10 +550,14 @@ srv_g_bivariate <- function(id,
       selector_list = selector_list,
       datasets = data
     )
+    qenv <- teal.code::eval_code(
+      data(),
+      'library("ggplot2");library("dplyr");library("teal.modules.general")' # nolint quotes
+    )
 
     anl_merged_q <- reactive({
       req(anl_merged_input())
-      data() %>%
+      qenv %>%
         teal.code::eval_code(as.expression(anl_merged_input()$expr))
     })
 
@@ -586,7 +620,7 @@ srv_g_bivariate <- function(id,
         }
       } else {
         shinyjs::hide("add_lines")
-        updateCheckboxInput(session, "add_lines", value = FALSE)
+        updateCheckboxInput(session, "add_lines", value = restoreInput(ns("add_lines"), FALSE))
         shinyjs::hide("alpha")
         shinyjs::hide("fixed_size")
         shinyjs::hide("size_settings")
@@ -650,33 +684,49 @@ srv_g_bivariate <- function(id,
         }
       }
 
-      # Add labels to facets
-      nulled_row_facet_name <- varname_w_label(row_facet_name, ANL)
-      nulled_col_facet_name <- varname_w_label(col_facet_name, ANL)
-      without_facet <- (is.null(nulled_row_facet_name) && is.null(nulled_col_facet_name)) || !facetting
-
-      print_call <- if (without_facet) {
-        quote(print(p))
-      } else {
-        substitute(
-          expr = {
-            # Add facetting labels
-            # optional: grid.newpage() # nolint: commented_code.
-            p <- add_facet_labels(p, xfacet_label = nulled_col_facet_name, yfacet_label = nulled_row_facet_name)
-            grid::grid.newpage()
-            grid::grid.draw(p)
-          },
-          env = list(nulled_col_facet_name = nulled_col_facet_name, nulled_row_facet_name = nulled_row_facet_name)
-        )
-      }
-
-      teal.code::eval_code(merged$anl_q_r(), substitute(expr = p <- cl, env = list(cl = cl))) %>%
-        teal.code::eval_code(print_call)
+      teal.code::eval_code(merged$anl_q_r(), substitute(expr = plot <- cl, env = list(cl = cl)))
     })
 
-    plot_r <- shiny::reactive({
-      output_q()[["p"]]
-    })
+    decorated_output_q_facets <- srv_decorate_teal_data(
+      "decorator",
+      data = output_q,
+      decorators = select_decorators(decorators, "plot"),
+      expr = reactive({
+        ANL <- merged$anl_q_r()[["ANL"]]
+        row_facet_name <- as.vector(merged$anl_input_r()$columns_source$row_facet)
+        col_facet_name <- as.vector(merged$anl_input_r()$columns_source$col_facet)
+
+        # Add labels to facets
+        nulled_row_facet_name <- varname_w_label(row_facet_name, ANL)
+        nulled_col_facet_name <- varname_w_label(col_facet_name, ANL)
+        facetting <- (isTRUE(input$facetting) && (!is.null(row_facet_name) || !is.null(col_facet_name)))
+        without_facet <- (is.null(nulled_row_facet_name) && is.null(nulled_col_facet_name)) || !facetting
+
+        print_call <- if (without_facet) {
+          quote(print(plot))
+        } else {
+          substitute(
+            expr = {
+              # Add facetting labels
+              # optional: grid.newpage() # nolint: commented_code.
+              # Prefixed with teal.modules.general as its usage will appear in "Show R code"
+              plot <- teal.modules.general::add_facet_labels(
+                plot,
+                xfacet_label = nulled_col_facet_name,
+                yfacet_label = nulled_row_facet_name
+              )
+              grid::grid.newpage()
+              grid::grid.draw(plot)
+            },
+            env = list(nulled_col_facet_name = nulled_col_facet_name, nulled_row_facet_name = nulled_row_facet_name)
+          )
+        }
+        print_call
+      }),
+      expr_is_reactive = TRUE
+    )
+
+    plot_r <- reactive(req(decorated_output_q_facets())[["plot"]])
 
     pws <- teal.widgets::plot_with_settings_srv(
       id = "myplot",
@@ -685,16 +735,13 @@ srv_g_bivariate <- function(id,
       width = plot_width
     )
 
-    teal.widgets::verbatim_popup_srv(
-      id = "warning",
-      verbatim_content = reactive(teal.code::get_warnings(output_q())),
-      title = "Warning",
-      disabled = reactive(is.null(teal.code::get_warnings(output_q())))
-    )
+    # Render R code.
+
+    source_code_r <- reactive(teal.code::get_code(req(decorated_output_q_facets())))
 
     teal.widgets::verbatim_popup_srv(
       id = "rcode",
-      verbatim_content = reactive(teal.code::get_code(output_q())),
+      verbatim_content = source_code_r,
       title = "Bivariate Plot"
     )
 
@@ -713,7 +760,7 @@ srv_g_bivariate <- function(id,
           card$append_text("Comment", "header3")
           card$append_text(comment)
         }
-        card$append_src(teal.code::get_code(output_q()))
+        card$append_src(source_code_r())
         card
       }
       teal.reporter::simple_reporter_srv("simple_reporter", reporter = reporter, card_fun = card_fun)
@@ -817,89 +864,89 @@ bivariate_ggplot_call <- function(x_class,
     Reduce(function(x, y) call("+", x, y), args)
   }
 
-  plot_call <- substitute(ggplot(data_name), env = list(data_name = as.name(data_name)))
+  plot_call <- substitute(ggplot2::ggplot(data_name), env = list(data_name = as.name(data_name)))
 
   # Single data plots
   if (x_class == "numeric" && y_class == "NULL") {
-    plot_call <- reduce_plot_call(plot_call, substitute(aes(x = xval), env = list(xval = x)))
+    plot_call <- reduce_plot_call(plot_call, substitute(ggplot2::aes(x = xval), env = list(xval = x)))
 
     if (freq) {
       plot_call <- reduce_plot_call(
         plot_call,
-        quote(geom_histogram(bins = 30)),
-        quote(ylab("Frequency"))
+        quote(ggplot2::geom_histogram(bins = 30)),
+        quote(ggplot2::ylab("Frequency"))
       )
     } else {
       plot_call <- reduce_plot_call(
         plot_call,
-        quote(geom_histogram(bins = 30, aes(y = after_stat(density)))),
-        quote(geom_density(aes(y = after_stat(density)))),
-        quote(ylab("Density"))
+        quote(ggplot2::geom_histogram(bins = 30, ggplot2::aes(y = ggplot2::after_stat(density)))),
+        quote(ggplot2::geom_density(ggplot2::aes(y = ggplot2::after_stat(density)))),
+        quote(ggplot2::ylab("Density"))
       )
     }
   } else if (x_class == "NULL" && y_class == "numeric") {
-    plot_call <- reduce_plot_call(plot_call, substitute(aes(x = yval), env = list(yval = y)))
+    plot_call <- reduce_plot_call(plot_call, substitute(ggplot2::aes(x = yval), env = list(yval = y)))
 
     if (freq) {
       plot_call <- reduce_plot_call(
         plot_call,
-        quote(geom_histogram(bins = 30)),
-        quote(ylab("Frequency"))
+        quote(ggplot2::geom_histogram(bins = 30)),
+        quote(ggplot2::ylab("Frequency"))
       )
     } else {
       plot_call <- reduce_plot_call(
         plot_call,
-        quote(geom_histogram(bins = 30, aes(y = after_stat(density)))),
-        quote(geom_density(aes(y = after_stat(density)))),
-        quote(ylab("Density"))
+        quote(ggplot2::geom_histogram(bins = 30, ggplot2::aes(y = ggplot2::after_stat(density)))),
+        quote(ggplot2::geom_density(ggplot2::aes(y = ggplot2::after_stat(density)))),
+        quote(ggplot2::ylab("Density"))
       )
     }
   } else if (x_class == "factor" && y_class == "NULL") {
-    plot_call <- reduce_plot_call(plot_call, substitute(aes(x = xval), env = list(xval = x)))
+    plot_call <- reduce_plot_call(plot_call, substitute(ggplot2::aes(x = xval), env = list(xval = x)))
 
     if (freq) {
       plot_call <- reduce_plot_call(
         plot_call,
-        quote(geom_bar()),
-        quote(ylab("Frequency"))
+        quote(ggplot2::geom_bar()),
+        quote(ggplot2::ylab("Frequency"))
       )
     } else {
       plot_call <- reduce_plot_call(
         plot_call,
-        quote(geom_bar(aes(y = after_stat(prop), group = 1))),
-        quote(ylab("Fraction"))
+        quote(ggplot2::geom_bar(ggplot2::aes(y = ggplot2::after_stat(prop), group = 1))),
+        quote(ggplot2::ylab("Fraction"))
       )
     }
   } else if (x_class == "NULL" && y_class == "factor") {
-    plot_call <- reduce_plot_call(plot_call, substitute(aes(x = yval), env = list(yval = y)))
+    plot_call <- reduce_plot_call(plot_call, substitute(ggplot2::aes(x = yval), env = list(yval = y)))
 
     if (freq) {
       plot_call <- reduce_plot_call(
         plot_call,
-        quote(geom_bar()),
-        quote(ylab("Frequency"))
+        quote(ggplot2::geom_bar()),
+        quote(ggplot2::ylab("Frequency"))
       )
     } else {
       plot_call <- reduce_plot_call(
         plot_call,
-        quote(geom_bar(aes(y = after_stat(prop), group = 1))),
-        quote(ylab("Fraction"))
+        quote(ggplot2::geom_bar(ggplot2::aes(y = ggplot2::after_stat(prop), group = 1))),
+        quote(ggplot2::ylab("Fraction"))
       )
     }
     # Numeric Plots
   } else if (x_class == "numeric" && y_class == "numeric") {
     plot_call <- reduce_plot_call(
       plot_call,
-      substitute(aes(x = xval, y = yval), env = list(xval = x, yval = y)),
+      substitute(ggplot2::aes(x = xval, y = yval), env = list(xval = x, yval = y)),
       # pch = 21 for consistent coloring behaviour b/w all geoms (outline and fill properties)
       `if`(
         !is.null(size),
         substitute(
-          geom_point(alpha = alphaval, size = sizeval, pch = 21),
+          ggplot2::geom_point(alpha = alphaval, size = sizeval, pch = 21),
           env = list(alphaval = alpha, sizeval = size)
         ),
         substitute(
-          geom_point(alpha = alphaval, pch = 21),
+          ggplot2::geom_point(alpha = alphaval, pch = 21),
           env = list(alphaval = alpha)
         )
       )
@@ -907,8 +954,8 @@ bivariate_ggplot_call <- function(x_class,
   } else if ((x_class == "numeric" && y_class == "factor") || (x_class == "factor" && y_class == "numeric")) {
     plot_call <- reduce_plot_call(
       plot_call,
-      substitute(aes(x = xval, y = yval), env = list(xval = x, yval = y)),
-      quote(geom_boxplot())
+      substitute(ggplot2::aes(x = xval, y = yval), env = list(xval = x, yval = y)),
+      quote(ggplot2::geom_boxplot())
     )
     # Factor and character plots
   } else if (x_class == "factor" && y_class == "factor") {
@@ -937,7 +984,7 @@ bivariate_ggplot_call <- function(x_class,
   dev_ggplot2_args <- teal.widgets::ggplot2_args(labs = labs_base)
 
   if (rotate_xaxis_labels) {
-    dev_ggplot2_args$theme <- list(axis.text.x = quote(element_text(angle = 45, hjust = 1)))
+    dev_ggplot2_args$theme <- list(axis.text.x = quote(ggplot2::element_text(angle = 45, hjust = 1)))
   }
 
   all_ggplot2_args <- teal.widgets::resolve_ggplot2_args(
@@ -1003,7 +1050,7 @@ coloring_ggplot_call <- function(colour,
       !identical(size, character(0))
   ) {
     substitute(
-      expr = aes(colour = colour_name, fill = fill_name, size = size_name),
+      expr = ggplot2::aes(colour = colour_name, fill = fill_name, size = size_name),
       env = list(colour_name = as.name(colour), fill_name = as.name(fill), size_name = as.name(size))
     )
   } else if (
@@ -1012,14 +1059,14 @@ coloring_ggplot_call <- function(colour,
       is_point &&
       identical(size, character(0))
   ) {
-    substitute(expr = aes(fill = fill_name), env = list(fill_name = as.name(fill)))
+    substitute(expr = ggplot2::aes(fill = fill_name), env = list(fill_name = as.name(fill)))
   } else if (
     !identical(colour, character(0)) &&
       !identical(fill, character(0)) &&
       (!is_point || identical(size, character(0)))
   ) {
     substitute(
-      expr = aes(colour = colour_name, fill = fill_name),
+      expr = ggplot2::aes(colour = colour_name, fill = fill_name),
       env = list(colour_name = as.name(colour), fill_name = as.name(fill))
     )
   } else if (
@@ -1027,20 +1074,20 @@ coloring_ggplot_call <- function(colour,
       identical(fill, character(0)) &&
       (!is_point || identical(size, character(0)))
   ) {
-    substitute(expr = aes(colour = colour_name), env = list(colour_name = as.name(colour)))
+    substitute(expr = ggplot2::aes(colour = colour_name), env = list(colour_name = as.name(colour)))
   } else if (
     identical(colour, character(0)) &&
       !identical(fill, character(0)) &&
       (!is_point || identical(size, character(0)))
   ) {
-    substitute(expr = aes(fill = fill_name), env = list(fill_name = as.name(fill)))
+    substitute(expr = ggplot2::aes(fill = fill_name), env = list(fill_name = as.name(fill)))
   } else if (
     identical(colour, character(0)) &&
       identical(fill, character(0)) &&
       is_point &&
       !identical(size, character(0))
   ) {
-    substitute(expr = aes(size = size_name), env = list(size_name = as.name(size)))
+    substitute(expr = ggplot2::aes(size = size_name), env = list(size_name = as.name(size)))
   } else if (
     !identical(colour, character(0)) &&
       identical(fill, character(0)) &&
@@ -1048,7 +1095,7 @@ coloring_ggplot_call <- function(colour,
       !identical(size, character(0))
   ) {
     substitute(
-      expr = aes(colour = colour_name, size = size_name),
+      expr = ggplot2::aes(colour = colour_name, size = size_name),
       env = list(colour_name = as.name(colour), size_name = as.name(size))
     )
   } else if (
@@ -1058,7 +1105,7 @@ coloring_ggplot_call <- function(colour,
       !identical(size, character(0))
   ) {
     substitute(
-      expr = aes(colour = colour_name, fill = fill_name, size = size_name),
+      expr = ggplot2::aes(colour = colour_name, fill = fill_name, size = size_name),
       env = list(colour_name = as.name(fill), fill_name = as.name(fill), size_name = as.name(size))
     )
   } else {
