@@ -39,10 +39,36 @@
 #' @note For more examples, please see the vignette "Using response plot" via
 #' `vignette("using-response-plot", package = "teal.modules.general")`.
 #'
+#' @section Decorating Module:
+#'
+#' This module generates the following objects, which can be modified in place using decorators:
+#' - `plot` (`ggplot`)
+#'
+#' A Decorator is applied to the specific output using a named list of `teal_transform_module` objects.
+#' The name of this list corresponds to the name of the output to which the decorator is applied.
+#' See code snippet below:
+#'
+#' ```
+#' tm_g_response(
+#'    ..., # arguments for module
+#'    decorators = list(
+#'      plot = teal_transform_module(...) # applied to the `plot` output
+#'    )
+#' )
+#' ```
+#'
+#' For additional details and examples of decorators, refer to the vignette
+#' `vignette("decorate-module-output", package = "teal.modules.general")`.
+#'
+#' To learn more please refer to the vignette
+#' `vignette("transform-module-output", package = "teal")` or the [`teal::teal_transform_module()`] documentation.
+#'
+#' @examplesShinylive
+#' library(teal.modules.general)
+#' interactive <- function() TRUE
+#' {{ next_example }}
 #' @examples
 #' # general data example
-#' library(teal.widgets)
-#'
 #' data <- teal_data()
 #' data <- within(data, {
 #'   require(nestcolor)
@@ -51,7 +77,6 @@
 #'     mtcars[[v]] <- as.factor(mtcars[[v]])
 #'   }
 #' })
-#' datanames(data) <- "mtcars"
 #'
 #' app <- init(
 #'   data = data,
@@ -77,9 +102,6 @@
 #'           multiple = FALSE,
 #'           fixed = FALSE
 #'         )
-#'       ),
-#'       ggplot2_args = ggplot2_args(
-#'         labs = list(subtitle = "Plot generated by Response Module")
 #'       )
 #'     )
 #'   )
@@ -88,16 +110,18 @@
 #'   shinyApp(app$ui, app$server)
 #' }
 #'
+#' @examplesShinylive
+#' library(teal.modules.general)
+#' interactive <- function() TRUE
+#' {{ next_example }}
+#' @examples
 #' # CDISC data example
-#' library(teal.widgets)
-#'
 #' data <- teal_data()
 #' data <- within(data, {
 #'   require(nestcolor)
-#'   ADSL <- rADSL
+#'   ADSL <- teal.data::rADSL
 #' })
-#' datanames(data) <- c("ADSL")
-#' join_keys(data) <- default_cdisc_join_keys[datanames(data)]
+#' join_keys(data) <- default_cdisc_join_keys[names(data)]
 #'
 #' app <- init(
 #'   data = data,
@@ -123,9 +147,6 @@
 #'           multiple = FALSE,
 #'           fixed = FALSE
 #'         )
-#'       ),
-#'       ggplot2_args = ggplot2_args(
-#'         labs = list(subtitle = "Plot generated by Response Module")
 #'       )
 #'     )
 #'   )
@@ -150,8 +171,10 @@ tm_g_response <- function(label = "Response Plot",
                           ggtheme = c("gray", "bw", "linedraw", "light", "dark", "minimal", "classic", "void"),
                           ggplot2_args = teal.widgets::ggplot2_args(),
                           pre_output = NULL,
-                          post_output = NULL) {
-  logger::log_info("Initializing tm_g_response")
+                          post_output = NULL,
+                          transformators = list(),
+                          decorators = list()) {
+  message("Initializing tm_g_response")
 
   # Normalize the parameters
   if (inherits(response, "data_extract_spec")) response <- list(response)
@@ -194,6 +217,8 @@ tm_g_response <- function(label = "Response Plot",
 
   checkmate::assert_multi_class(pre_output, c("shiny.tag", "shiny.tag.list", "html"), null.ok = TRUE)
   checkmate::assert_multi_class(post_output, c("shiny.tag", "shiny.tag.list", "html"), null.ok = TRUE)
+
+  assert_decorators(decorators, "plot")
   # End of assertions
 
   # Make UI args
@@ -206,17 +231,25 @@ tm_g_response <- function(label = "Response Plot",
     col_facet = col_facet
   )
 
-  module(
+  ans <- module(
     label = label,
     server = srv_g_response,
     ui = ui_g_response,
     ui_args = args,
     server_args = c(
       data_extract_list,
-      list(plot_height = plot_height, plot_width = plot_width, ggplot2_args = ggplot2_args)
+      list(
+        plot_height = plot_height,
+        plot_width = plot_width,
+        ggplot2_args = ggplot2_args,
+        decorators = decorators
+      )
     ),
+    transformators = transformators,
     datanames = teal.transform::get_extract_datanames(data_extract_list)
   )
+  attr(ans, "teal_bookmarkable") <- TRUE
+  ans
 }
 
 # UI function for the response module
@@ -229,7 +262,7 @@ ui_g_response <- function(id, ...) {
     output = teal.widgets::white_small_well(
       teal.widgets::plot_with_settings_ui(id = ns("myplot"))
     ),
-    encoding = div(
+    encoding = tags$div(
       ### Reporter
       teal.reporter::simple_reporter_ui(ns("simple_reporter")),
       ###
@@ -270,6 +303,7 @@ ui_g_response <- function(id, ...) {
         selected = ifelse(args$freq, "frequency", "density"),
         justified = TRUE
       ),
+      ui_decorate_teal_data(ns("decorator"), decorators = select_decorators(args$decorators, "plot")),
       teal.widgets::panel_group(
         teal.widgets::panel_item(
           title = "Plot settings",
@@ -287,7 +321,6 @@ ui_g_response <- function(id, ...) {
       )
     ),
     forms = tagList(
-      teal.widgets::verbatim_popup_ui(ns("warning"), button_label = "Show Warnings"),
       teal.widgets::verbatim_popup_ui(ns("rcode"), "Show R code")
     ),
     pre_output = args$pre_output,
@@ -306,12 +339,15 @@ srv_g_response <- function(id,
                            col_facet,
                            plot_height,
                            plot_width,
-                           ggplot2_args) {
+                           ggplot2_args,
+                           decorators) {
   with_reporter <- !missing(reporter) && inherits(reporter, "Reporter")
   with_filter <- !missing(filter_panel_api) && inherits(filter_panel_api, "FilterPanelAPI")
   checkmate::assert_class(data, "reactive")
   checkmate::assert_class(isolate(data()), "teal_data")
   moduleServer(id, function(input, output, session) {
+    teal.logger::log_shiny_input_changes(input, namespace = "teal.modules.general")
+
     data_extract <- list(response = response, x = x, row_facet = row_facet, col_facet = col_facet)
 
     rule_diff <- function(other) {
@@ -357,9 +393,14 @@ srv_g_response <- function(id,
       datasets = data
     )
 
+    qenv <- teal.code::eval_code(
+      data(),
+      'library("ggplot2");library("dplyr")' # nolint quotes
+    )
+
     anl_merged_q <- reactive({
       req(anl_merged_input())
-      data() %>%
+      qenv %>%
         teal.code::eval_code(as.expression(anl_merged_input()$expr))
     })
 
@@ -443,8 +484,8 @@ srv_g_response <- function(id,
         )
 
       plot_call <- substitute(
-        expr = ggplot(ANL2, aes(x = x_cl, y = ns)) +
-          geom_bar(aes(fill = resp_cl), stat = "identity", position = arg_position),
+        expr = ggplot2::ggplot(ANL2, ggplot2::aes(x = x_cl, y = ns)) +
+          ggplot2::geom_bar(ggplot2::aes(fill = resp_cl), stat = "identity", position = arg_position),
         env = list(
           x_cl = x_cl,
           resp_cl = resp_cl,
@@ -452,21 +493,26 @@ srv_g_response <- function(id,
         )
       )
 
-      if (!freq) plot_call <- substitute(plot_call + expand_limits(y = c(0, 1.1)), env = list(plot_call = plot_call))
+      if (!freq) {
+        plot_call <- substitute(
+          plot_call + ggplot2::expand_limits(y = c(0, 1.1)),
+          env = list(plot_call = plot_call)
+        )
+      }
 
       if (counts) {
         plot_call <- substitute(
           expr = plot_call +
-            geom_text(
+            ggplot2::geom_text(
               data = ANL2,
-              aes(label = ns, x = x_cl, y = ns, group = resp_cl),
+              ggplot2::aes(label = ns, x = x_cl, y = ns, group = resp_cl),
               col = "white",
               vjust = "middle",
               hjust = "middle",
               position = position_anl2_value
             ) +
-            geom_text(
-              data = ANL3, aes(label = ns, x = x_cl, y = anl3_y),
+            ggplot2::geom_text(
+              data = ANL3, ggplot2::aes(label = ns, x = x_cl, y = anl3_y),
               hjust = hjust_value,
               vjust = vjust_value,
               position = position_anl3_value
@@ -477,7 +523,7 @@ srv_g_response <- function(id,
             resp_cl = resp_cl,
             hjust_value = if (swap_axes) "left" else "middle",
             vjust_value = if (swap_axes) "middle" else -1,
-            position_anl2_value = if (!freq) quote(position_fill(0.5)) else quote(position_stack(0.5)),
+            position_anl2_value = if (!freq) quote(position_fill(0.5)) else quote(position_stack(0.5)), # nolint: line_length.
             anl3_y = if (!freq) 1.1 else as.name("ns"),
             position_anl3_value = if (!freq) "fill" else "stack"
           )
@@ -504,7 +550,7 @@ srv_g_response <- function(id,
       )
 
       if (rotate_xaxis_labels) {
-        dev_ggplot2_args$theme[["axis.text.x"]] <- quote(element_text(angle = 45, hjust = 1))
+        dev_ggplot2_args$theme[["axis.text.x"]] <- quote(ggplot2::element_text(angle = 45, hjust = 1))
       }
 
       all_ggplot2_args <- teal.widgets::resolve_ggplot2_args(
@@ -518,8 +564,7 @@ srv_g_response <- function(id,
       )
 
       plot_call <- substitute(expr = {
-        p <- plot_call + labs + ggthemes + themes
-        print(p)
+        plot <- plot_call + labs + ggthemes + themes
       }, env = list(
         plot_call = plot_call,
         labs = parsed_ggplot2_args$labs,
@@ -530,7 +575,14 @@ srv_g_response <- function(id,
       teal.code::eval_code(qenv, plot_call)
     })
 
-    plot_r <- reactive(output_q()[["p"]])
+    decorated_output_plot_q <- srv_decorate_teal_data(
+      id = "decorator",
+      data = output_q,
+      decorators = select_decorators(decorators, "plot"),
+      expr = print(plot)
+    )
+
+    plot_r <- reactive(req(decorated_output_plot_q())[["plot"]])
 
     # Insert the plot into a plot_with_settings module from teal.widgets
     pws <- teal.widgets::plot_with_settings_srv(
@@ -540,16 +592,12 @@ srv_g_response <- function(id,
       width = plot_width
     )
 
-    teal.widgets::verbatim_popup_srv(
-      id = "warning",
-      verbatim_content = reactive(teal.code::get_warnings(output_q())),
-      title = "Warning",
-      disabled = reactive(is.null(teal.code::get_warnings(output_q())))
-    )
+    # Render R code.
+    source_code_r <- reactive(teal.code::get_code(req(decorated_output_plot_q())))
 
     teal.widgets::verbatim_popup_srv(
       id = "rcode",
-      verbatim_content = reactive(teal.code::get_code(output_q())),
+      verbatim_content = source_code_r,
       title = "Show R Code for Response"
     )
 
@@ -568,7 +616,7 @@ srv_g_response <- function(id,
           card$append_text("Comment", "header3")
           card$append_text(comment)
         }
-        card$append_src(teal.code::get_code(output_q()))
+        card$append_src(source_code_r())
         card
       }
       teal.reporter::simple_reporter_srv("simple_reporter", reporter = reporter, card_fun = card_fun)
