@@ -30,16 +30,44 @@
 #'
 #' @inherit shared_params return
 #'
-#' @examples
-#' library(teal.widgets)
+#' @section Decorating Module:
 #'
+#' This module generates the following objects, which can be modified in place using decorators:
+#' - `plot` (`ggplot`)
+#'
+#' A Decorator is applied to the specific output using a named list of `teal_transform_module` objects.
+#' The name of this list corresponds to the name of the output to which the decorator is applied.
+#' See code snippet below:
+#'
+#' ```
+#' tm_g_scatterplot(
+#'    ..., # arguments for module
+#'    decorators = list(
+#'      plot = teal_transform_module(...) # applied to the `plot` output
+#'    )
+#' )
+#' ```
+#'
+#' For additional details and examples of decorators, refer to the vignette
+#' `vignette("decorate-module-output", package = "teal.modules.general")`.
+#'
+#' To learn more please refer to the vignette
+#' `vignette("transform-module-output", package = "teal")` or the [`teal::teal_transform_module()`] documentation.
+#'
+#'
+#' @examplesShinylive
+#' library(teal.modules.general)
+#' interactive <- function() TRUE
+#' {{ next_example }}
+# nolint start: line_length_linter.
+#' @examples
+# nolint end: line_length_linter.
 #' # general data example
 #' data <- teal_data()
 #' data <- within(data, {
 #'   require(nestcolor)
 #'   CO2 <- CO2
 #' })
-#' datanames(data) <- "CO2"
 #'
 #' app <- init(
 #'   data = data,
@@ -108,9 +136,6 @@
 #'           multiple = FALSE,
 #'           fixed = FALSE
 #'         )
-#'       ),
-#'       ggplot2_args = ggplot2_args(
-#'         labs = list(subtitle = "Plot generated by Scatterplot Module")
 #'       )
 #'     )
 #'   )
@@ -119,14 +144,20 @@
 #'   shinyApp(app$ui, app$server)
 #' }
 #'
+#' @examplesShinylive
+#' library(teal.modules.general)
+#' interactive <- function() TRUE
+#' {{ next_example }}
+# nolint start: line_length_linter.
+#' @examples
+# nolint end: line_length_linter.
 #' # CDISC data example
 #' data <- teal_data()
 #' data <- within(data, {
 #'   require(nestcolor)
-#'   ADSL <- rADSL
+#'   ADSL <- teal.data::rADSL
 #' })
-#' datanames(data) <- c("ADSL")
-#' join_keys(data) <- default_cdisc_join_keys[datanames(data)]
+#' join_keys(data) <- default_cdisc_join_keys[names(data)]
 #'
 #' app <- init(
 #'   data = data,
@@ -195,9 +226,6 @@
 #'           multiple = FALSE,
 #'           fixed = FALSE
 #'         )
-#'       ),
-#'       ggplot2_args = ggplot2_args(
-#'         labs = list(subtitle = "Plot generated by Scatterplot Module")
 #'       )
 #'     )
 #'   )
@@ -226,18 +254,10 @@ tm_g_scatterplot <- function(label = "Scatterplot",
                              pre_output = NULL,
                              post_output = NULL,
                              table_dec = 4,
-                             ggplot2_args = teal.widgets::ggplot2_args()) {
-  logger::log_info("Initializing tm_g_scatterplot")
-
-  # Requires Suggested packages
-  extra_packages <- c("ggpmisc", "ggExtra", "colourpicker")
-  missing_packages <- Filter(function(x) !requireNamespace(x, quietly = TRUE), extra_packages)
-  if (length(missing_packages) > 0L) {
-    stop(sprintf(
-      "Cannot load package(s): %s.\nInstall or restart your session.",
-      toString(missing_packages)
-    ))
-  }
+                             ggplot2_args = teal.widgets::ggplot2_args(),
+                             transformators = list(),
+                             decorators = list()) {
+  message("Initializing tm_g_scatterplot")
 
   # Normalize the parameters
   if (inherits(x, "data_extract_spec")) x <- list(x)
@@ -294,6 +314,9 @@ tm_g_scatterplot <- function(label = "Scatterplot",
 
   checkmate::assert_scalar(table_dec)
   checkmate::assert_class(ggplot2_args, "ggplot2_args")
+
+  assert_decorators(decorators, "plot")
+
   # End of assertions
 
   # Make UI args
@@ -308,17 +331,26 @@ tm_g_scatterplot <- function(label = "Scatterplot",
     col_facet = col_facet
   )
 
-  module(
+  ans <- module(
     label = label,
     server = srv_g_scatterplot,
     ui = ui_g_scatterplot,
     ui_args = args,
     server_args = c(
       data_extract_list,
-      list(plot_height = plot_height, plot_width = plot_width, table_dec = table_dec, ggplot2_args = ggplot2_args)
+      list(
+        plot_height = plot_height,
+        plot_width = plot_width,
+        table_dec = table_dec,
+        ggplot2_args = ggplot2_args,
+        decorators = decorators
+      )
     ),
+    transformators = transformators,
     datanames = teal.transform::get_extract_datanames(data_extract_list)
   )
+  attr(ans, "teal_bookmarkable") <- TRUE
+  ans
 }
 
 # UI function for the scatterplot module
@@ -329,7 +361,7 @@ ui_g_scatterplot <- function(id, ...) {
     args$x, args$y, args$color_by, args$size_by, args$row_facet, args$col_facet
   )
 
-  shiny::tagList(
+  tagList(
     include_css_files("custom"),
     teal.widgets::standard_layout(
       output = teal.widgets::white_small_well(
@@ -338,7 +370,7 @@ ui_g_scatterplot <- function(id, ...) {
         teal.widgets::get_dt_rows(ns("data_table"), ns("data_table_rows")),
         DT::dataTableOutput(ns("data_table"), width = "100%")
       ),
-      encoding = div(
+      encoding = tags$div(
         ### Reporter
         teal.reporter::simple_reporter_ui(ns("simple_reporter")),
         ###
@@ -408,6 +440,7 @@ ui_g_scatterplot <- function(id, ...) {
             is_single_dataset = is_single_dataset_value
           )
         },
+        ui_decorate_teal_data(ns("decorator"), decorators = select_decorators(args$decorators, "plot")),
         teal.widgets::panel_group(
           teal.widgets::panel_item(
             title = "Plot settings",
@@ -432,11 +465,11 @@ ui_g_scatterplot <- function(id, ...) {
             shinyjs::hidden(checkboxInput(ns("show_form"), "Show formula", value = TRUE)),
             shinyjs::hidden(checkboxInput(ns("show_r2"), "Show adj-R Squared", value = TRUE)),
             uiOutput(ns("num_na_removed")),
-            div(
+            tags$div(
               id = ns("label_pos"),
-              div(strong("Stats position")),
-              div(class = "inline-block w-10", helpText("Left")),
-              div(
+              tags$div(tags$strong("Stats position")),
+              tags$div(class = "inline-block w-10", helpText("Left")),
+              tags$div(
                 class = "inline-block w-70",
                 teal.widgets::optionalSliderInput(
                   ns("pos"),
@@ -444,7 +477,7 @@ ui_g_scatterplot <- function(id, ...) {
                   min = 0, max = 1, value = .99, ticks = FALSE, step = .01
                 )
               ),
-              div(class = "inline-block w-10", helpText("Right"))
+              tags$div(class = "inline-block w-10", helpText("Right"))
             ),
             teal.widgets::optionalSliderInput(
               ns("label_size"), "Stats font size",
@@ -464,7 +497,6 @@ ui_g_scatterplot <- function(id, ...) {
         )
       ),
       forms = tagList(
-        teal.widgets::verbatim_popup_ui(ns("warning"), button_label = "Show Warnings"),
         teal.widgets::verbatim_popup_ui(ns("rcode"), "Show R code")
       ),
       pre_output = args$pre_output,
@@ -487,12 +519,15 @@ srv_g_scatterplot <- function(id,
                               plot_height,
                               plot_width,
                               table_dec,
-                              ggplot2_args) {
+                              ggplot2_args,
+                              decorators) {
   with_reporter <- !missing(reporter) && inherits(reporter, "Reporter")
   with_filter <- !missing(filter_panel_api) && inherits(filter_panel_api, "FilterPanelAPI")
   checkmate::assert_class(data, "reactive")
   checkmate::assert_class(isolate(data()), "teal_data")
   moduleServer(id, function(input, output, session) {
+    teal.logger::log_shiny_input_changes(input, namespace = "teal.modules.general")
+
     data_extract <- list(
       x = x,
       y = y,
@@ -554,10 +589,11 @@ srv_g_scatterplot <- function(id,
       datasets = data,
       merge_function = "dplyr::inner_join"
     )
+    qenv <- teal.code::eval_code(data(), 'library("ggplot2");library("dplyr")') # nolint quotes
 
     anl_merged_q <- reactive({
       req(anl_merged_input())
-      data() %>%
+      qenv %>%
         teal.code::eval_code(as.expression(anl_merged_input()$expr)) %>%
         teal.code::eval_code(quote(ANL)) # used to display table when running show-r-code code
     })
@@ -599,7 +635,7 @@ srv_g_scatterplot <- function(id,
         x_var <- as.vector(merged$anl_input_r()$columns_source$x)
         y_var <- as.vector(merged$anl_input_r()$columns_source$y)
         if ((num_total_na <- nrow(ANL) - nrow(stats::na.omit(ANL[, c(x_var, y_var)]))) > 0) {
-          shiny::tags$div(paste(num_total_na, "row(s) with missing values were removed"), shiny::tags$hr())
+          tags$div(paste(num_total_na, "row(s) with missing values were removed"), tags$hr())
         }
       }
     })
@@ -766,7 +802,7 @@ srv_g_scatterplot <- function(id,
         "ANL"
       }
 
-      plot_call <- substitute(expr = pre_pro_anl %>% ggplot(), env = list(pre_pro_anl = str2lang(pre_pro_anl)))
+      plot_call <- substitute(expr = pre_pro_anl %>% ggplot2::ggplot(), env = list(pre_pro_anl = str2lang(pre_pro_anl)))
 
       plot_call <- if (length(color_by_var) == 0) {
         substitute(
@@ -934,7 +970,7 @@ srv_g_scatterplot <- function(id,
       )
 
       if (rotate_xaxis_labels) {
-        dev_ggplot2_args$theme[["axis.text.x"]] <- quote(element_text(angle = 45, hjust = 1))
+        dev_ggplot2_args$theme[["axis.text.x"]] <- quote(ggplot2::element_text(angle = 45, hjust = 1))
       }
 
       all_ggplot2_args <- teal.widgets::resolve_ggplot2_args(
@@ -975,13 +1011,19 @@ srv_g_scatterplot <- function(id,
         )
       }
 
-      plot_call <- substitute(expr = p <- plot_call, env = list(plot_call = plot_call))
+      plot_call <- substitute(expr = plot <- plot_call, env = list(plot_call = plot_call))
 
-      teal.code::eval_code(plot_q, plot_call) %>%
-        teal.code::eval_code(quote(print(p)))
+      teal.code::eval_code(plot_q, plot_call)
     })
 
-    plot_r <- reactive(output_q()[["p"]])
+    decorated_output_plot_q <- srv_decorate_teal_data(
+      id = "decorator",
+      data = output_q,
+      decorators = select_decorators(decorators, "plot"),
+      expr = print(plot)
+    )
+
+    plot_r <- reactive(req(decorated_output_plot_q())[["plot"]])
 
     # Insert the plot into a plot_with_settings module from teal.widgets
     pws <- teal.widgets::plot_with_settings_srv(
@@ -1020,16 +1062,12 @@ srv_g_scatterplot <- function(id,
       }
     })
 
-    teal.widgets::verbatim_popup_srv(
-      id = "warning",
-      verbatim_content = reactive(teal.code::get_warnings(output_q())),
-      title = "Warning",
-      disabled = reactive(is.null(teal.code::get_warnings(output_q())))
-    )
+    # Render R code.
+    source_code_r <- reactive(teal.code::get_code(req(decorated_output_plot_q())))
 
     teal.widgets::verbatim_popup_srv(
       id = "rcode",
-      verbatim_content = reactive(teal.code::get_code(output_q())),
+      verbatim_content = source_code_r,
       title = "R Code for scatterplot"
     )
 
@@ -1048,7 +1086,7 @@ srv_g_scatterplot <- function(id,
           card$append_text("Comment", "header3")
           card$append_text(comment)
         }
-        card$append_src(teal.code::get_code(output_q()))
+        card$append_src(source_code_r())
         card
       }
       teal.reporter::simple_reporter_srv("simple_reporter", reporter = reporter, card_fun = card_fun)
